from pyIcingaFramework.check import IcingaCheck, Measurement
from pyIcingaAPIChecks.rest import RestClient

from requests.auth import HTTPBasicAuth
import logging

from .intersight_auth import IntersightAuth

class Alarms(IcingaCheck):
    def check(self, **kwargs):

        apikey = self.extra_options.get('api-key', None)
        certfile = self.extra_options.get('api-cert-file', None)

        self.set_threshold('*', '1', '3')

        auth = IntersightAuth(
            api_key_id=apikey,
            secret_key_filename=certfile
        )

        logging.debug(self.extra_options)
        client = RestClient(self.host, int(self.port), '/api/v1', verify=False, auth=auth)

        endpoint = client.endpoint('/cond/Alarms?$filter=Severity ne Cleared')
        resp = endpoint.get()

        logging.debug(f'API Call returned {resp.status_code}')

        active_alarm_severities = {'none': 0, 'info': 0, 'warning': 0, 'critical': 0}

        if resp.status_code == 200:
            for alarm in resp.Results:
                active_alarm_severities[alarm.Severity.lower()] += 1
        else:
            logging.info(endpoint._response.text)


        self.result.add_measurement('info_alarms', active_alarm_severities['info'],
                                    self.get_threshold('warning', 'info_alarms'),
                                    self.get_threshold('critical', 'info_alarms'),
                                    alerts=False)
        self.result.add_measurement('warning_alarms', active_alarm_severities['warning'],
                                    self.get_threshold('warning', 'warning_alarms'),
                                    self.get_threshold('critical', 'warning_alarms'),
                                    alerts=True)
        self.result.add_measurement('critical_alarms', active_alarm_severities['critical'],
                                    self.get_threshold('warning', 'warning_alarms'),
                                    self.get_threshold('critical', 'warning_alarms'),
                                    alerts=True)

        status_code = 0

        # Set stdout messages
        logging.info('checking thresholds...')
        for measurement in self.result.measurements:
            logging.debug('check measurement %s for breaches' % measurement.label)
            statusCode = measurement.status()
            if statusCode > status_code:
                status_code = int(statusCode)

        logging.info('Creating stdout msg...')
        output = []
        output.append(f'[{Measurement.status_text(status_code)}] Intersight Alarm Status')
        for measurement in self.result.measurements:
            output.append(f'\_ [{Measurement.status_text(measurement.status())}] {measurement.label.replace("_alarms", "").title()}: {measurement.value}') # noqa

        self.result.stdout = '\n'.join(output)

        return self.result



import requests
import urllib3
import logging
from requests.exceptions import ProxyError


from json.decoder import JSONDecodeError

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
"""
A restful client that returns a response for any json type rest API
"""
class RestClient(object):

    def __init__(self, host, port=None, base_uri='/', use_ssl=True, auth=None, verify=True):

        if type(verify) is not bool:
            raise ValueError("Verify SSL must be of type bool")

        if type(port) is not int:
            raise ValueError("Port must be of type int")

        self.host = host
        
        if not port and use_ssl:
            self.port = 443
        elif not port and not use_ssl:
            self.port = 80
        else:
            self.port = port
        
        self.base_uri = base_uri
        self.auth = auth

        self.ssl_verify = verify
        self.use_ssl = use_ssl

        
        self.base_url = '%s:%s%s' % (self.host, self.port, self.base_uri)

        if use_ssl:
            self.base_url = 'https://%s' % self.base_url
        else:
            self.base_url = 'http://%s' % self.base_url

        self.session = None
        self.session = self._getSession()

    def _getSession(self):
        """ Create a session using basic auth

        :return:
            - :class:`requests.Session` object
        """
        if not self.session:
            s = requests.session()

            if self.auth != None:
                s.auth = self.auth
        else:
            s = self.session

        return s

    def endpoint(self, endpoint):
        """ Creates a new endpoint
        :param api_path: Path to the API 
        :return:
            - :class:`Endpoint` object
        
        """
        return Endpoint(self.base_url, endpoint, self.session, verify=self.ssl_verify)


"""
API JSON Repsonse
"""


class JSONResponse(object):

    def __init__(self, status_code, content):
        self._current_item = 0
        self._status_code = status_code
        self._itemlist = []
        if isinstance(content, list):
            content = {'results': content}

        self.raw = content
        if isinstance(content, dict):
            self._parse_result(content)
            #logging.info(self._itemlist)

    @property
    def status_code(self) -> int:
        '''Returns the status code of the request for the endpoint
           :param content: JSON response from the API Call
           :return:
              - :int: Returns the status code of the api call
        '''
        return self._status_code

    def __iter__(self):
        for i in dict(self._itemlist):
            c = getattr(self, i)
            if isinstance(c, JSONResponse):
                yield i, dict(c)

            elif isinstance(c, list) and all(
                isinstance(i, JSONDecodeError) for i in c
            ):
                yield i, [ dict(x) for x in c ]
            else:
                yield i, c


    def _parse_result(self, content):
        '''Parses the json respnse of the rest client and builds a class object for each time'''
        try:
            if isinstance(content, dict):
                for k, v in content.items():
                    if isinstance(v,dict):
                        v = JSONResponse(self._status_code, v)

                        self._itemlist.append((k, v))
                    elif isinstance(v, list):
                        v = [ JSONResponse(self.status_code, x) for x in v ]
                        self._itemlist.append((k,list(v)))
                    else:
                        self._itemlist.append((k, v))
                        
                    setattr(self, k, v)
        except JSONDecodeError:
            pass

"""
Class that represents an API endpoint response.

"""
class Endpoint(object):
    def __init__(self, base_url, endpoint_path, session, verify=True, **kwargs):
        self.base_url = base_url
        self.endpoint_path = endpoint_path
        self.session = session

        self.endpoint = '%s%s' % (self.base_url, self.endpoint_path)
        self.verify = verify
        self.kwargs = kwargs
        self._response = None

    @property
    def path(self):
        return '%s' % self.base_url

    def _request(self,output, headers=None):
        logging.debug('getting endpoint: {}'.format(self.endpoint))
        try:
            if headers and not type(headers) is dict:
                headers = {}
            self._response = self.session.get(self.endpoint, verify=self.verify, headers=headers)
        except ProxyError as pe:
            logging.debug('Proxy Error: %r' % pe.__dict__.keys())
            logging.debug('Proxy Error response: %r' % pe.response)
            self._response = pe.response

        if not self._response:
            return JSONResponse(503, {})

        logging.debug('Json Response: {}'.format(self._response.__dict__.get('json', None)))
        if output == 'json':
            try: 
                logging.debug('Building JSON Response')
                return JSONResponse(self._response.status_code, self._response.json())
            except ValueError:
                return self._response
        else:
            logging.debug('Returning Response')
            return self._response

        return None

    def get(self, output='json', headers=None):
        return self._request(output, headers=headers)



"""
    intersight_auth.py -  provides a class to support Cisco Intersight
    interactions

    author: Chris Gascoigne (cgascoig@cisco.com)

MIT License

Copyright (c) 2020 Cisco DevNet

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. 
"""

# pylint: disable=too-few-public-methods
from base64 import b64encode
from email.utils import formatdate
from six.moves.urllib.parse import urlparse

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import padding

from requests.auth import AuthBase

def _get_sha256_digest(data):

    hasher = hashes.Hash(hashes.SHA256(), default_backend())

    if data is not None:
        hasher.update(data.encode())

    return hasher.finalize()


def _prepare_string_to_sign(req_tgt, hdrs):
    """
    :param req_tgt : Request Target as stored in http header.
    :param hdrs: HTTP Headers to be signed.
    :return: instance of digest object
    """

    signature_string = '(request-target): ' + req_tgt.lower() + '\n'

    for i, (key, value) in enumerate(hdrs.items()):
        signature_string += key.lower() + ': ' + value
        if i < len(hdrs.items())-1:
            signature_string += '\n'

    return signature_string


def _get_rsasig_b64(key, string_to_sign):

    return b64encode(key.sign(
        string_to_sign,
        padding.PKCS1v15(),
        hashes.SHA256()))


def _get_auth_header(signing_headers, method, path, api_key_id, secret_key):

    string_to_sign = _prepare_string_to_sign(method + " " + path, signing_headers)
    b64_signed_auth_digest = _get_rsasig_b64(secret_key, string_to_sign.encode())

    auth_str = (
        'Signature keyId="' + api_key_id + '",' +
        'algorithm="rsa-sha256",headers="(request-target)'
        )

    for key in signing_headers:
        auth_str += ' ' + key.lower()

    auth_str += (
        '", signature="' + b64_signed_auth_digest.decode('ascii') + '"'
        )

    return auth_str


class IntersightAuth(AuthBase):
    """Implements requests custom authentication for Cisco Intersight"""

    def __init__(self, secret_key_filename, api_key_id, secret_key_file_password=None):
        self.secret_key_filename = secret_key_filename
        self.api_key_id = api_key_id
        self.secret_key_file_password = secret_key_file_password

        with open(secret_key_filename, "rb") as secret_key_file:
            self.secret_key = serialization.load_pem_private_key(
                secret_key_file.read(),
                password=secret_key_file_password,
                backend=default_backend()
                )

    def __call__(self, r):
        """Called by requests to modify and return the authenticated request"""
        date = formatdate(timeval=None, localtime=False, usegmt=True)
        # date = "Tue, 07 Aug 2018 04:03:47 GMT"

        digest = _get_sha256_digest(r.body)

        url = urlparse(r.url)
        path = url.path or "/"
        if url.query:
            path += "?"+url.query

        signing_headers = {
            "Date": date,
            "Host": url.hostname,
            "Content-Type": r.headers.get('Content-Type') or "application/json",
            "Digest": "SHA-256=%s" % b64encode(digest).decode('ascii'),
        }

        auth_header = _get_auth_header(
            signing_headers, r.method, path, self.api_key_id, self.secret_key)

        r.headers['Digest'] = "SHA-256=%s" % b64encode(digest).decode('ascii')
        r.headers['Date'] = date
        r.headers['Authorization'] = "%s" % auth_header
        r.headers['Host'] = url.hostname
        r.headers['Content-Type'] = signing_headers['Content-Type']

        return r


from pyIcingaFramework.check import IcingaCheck, Measurement
from pyIcingaAPIChecks.rest import RestClient

from requests.auth import HTTPBasicAuth
import logging

from .intersight_auth import IntersightAuth

class NodeStatus(IcingaCheck):
    def check(self, **kwargs):

        apikey = self.extra_options.get('api-key', None)
        certfile = self.extra_options.get('api-cert-file', None)

        # set default thresholds
        self.set_threshold('80', '90', label='*')
        self.set_threshold('%Prepairing', '%Down', label='node_status')
        self.set_threshold('~%Impaired;;Operational', '%Impaired', label='op_status')

        auth = IntersightAuth(
            api_key_id=apikey,
            secret_key_filename=certfile
        )

        moid = self.filters.get('moid', None)
        logging.debug(f'Filter on Moid: {moid}')

        logging.debug(self.extra_options)
        client = RestClient(self.host, int(self.port), '/api/v1', verify=False, auth=auth)

        endpoint = client.endpoint('/appliance/NodeStatuses')
        resp = endpoint.get()

        logging.debug(f'API Call returned {resp.status_code}')

        if resp.status_code == 200:
            logging.debug('------- API RETURN Output ------')
            for r in resp.Results:
                logging.debug(f'Node State: {r.NodeState}')
                self.result.add_measurement('Node State', r.NodeState, warning=self.get_threshold('warning', 'node_status'), critical=self.get_threshold('critical', 'node_status'), is_perf=False)
                logging.debug(f'Node State: {r.OperationalStatus}')
                self.result.add_measurement('Operational Status', r.OperationalStatus, warning=self.get_threshold('warning', 'op_status'), critical=self.get_threshold('critical', 'op_status'), is_perf=False)

                logging.debug(f'cpu: {r.CpuUsage}%')
                self.result.add_measurement('load', r.CpuUsage, uom='%', warning=self.get_threshold('warning', 'load'), critical=self.get_threshold('critical', 'load'))
                logging.debug(f'mem: {r.MemUsage}%')
                self.result.add_measurement('memory_usage', r.MemUsage, uom='%', warning=self.get_threshold('warning', 'memory_usage'), critical=self.get_threshold('critical', 'memory_usage'))


                for status in r.StatusChecks:
                    logging.info(f'{status.Code}: {status.Result}')
                    self.result.add_measurement(f'status_{status.Code.lower()}', status.Result.lower(),  is_perf=False, alerts=False)

        else:
            logging.info(endpoint._response.text)


        status_code = 0

        # Set stdout messages
        logging.info('checking thresholds...')
        for measurement in self.result.measurements:
            logging.debug('check measurement %s for breaches' % measurement.label)
            statusCode = measurement.status()
            if statusCode > status_code:
                status_code = int(statusCode)

        logging.info('Creating stdout msg...')
        output = []
        output.append(f'[{Measurement.status_text(status_code)}] Intersight Node Status')
        for measurement in self.result.measurements:
            if measurement.label.startswith('status_') is False:
                output.append(f'\_ [{Measurement.status_text(measurement.status())}] {measurement.label.replace("_", " ").title()}: {measurement.value} {measurement.unitOfMeasure}') # noqa
        output.append('\_ Status Checks')
        for measurement in self.result.measurements:
            if measurement.label.startswith('status_') is True:
                output.append(f'   \_ [{Measurement.status_text(measurement.status())}] {measurement.label.replace("status_node_", "node ").title()}: {measurement.value}') # noqa

        self.result.stdout = '\n'.join(output)

        return self.result


from setuptools import setup, find_packages


setup(
    name='pyIcingaAPIChecks',
    description='Icinga2 Check API Checks for pyIcingaFramework',
    long_description="# Icinga2 Check API Checks for pyIcingaFramework",
    version='0.0.5',
    author='Paul Denning',
    include_package_data=True,
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=[
        "pyIcingaFramework",
    ],
    keywords=['icinga2', 'nagios'],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
    ],
)